import os, time, psutil
from datetime import datetime
from typing import Tuple
from httpx import AsyncClient


_STATUS_MSG = """
> Status Overview

[CPU: {b_cpu}% of {p_cpu}%]
[Memory: {b_mem} of {p_mem}%]
[Disk usage: {p_disk}%]

[Baidu: {baidu_}]
[Google: {google_}]

[Net sent: {inteSENT}MB]
[Net recv: {inteRECV}MB]

[Run Duration]
[Bot: {bot_time}]
[Platform: {boot_time}]
{msg}
""".strip()


async def get_status() -> Tuple[str, bool]:
    try:
        cpu = psutil.cpu_percent(interval=1)
        mem = psutil.virtual_memory().percent
        disk = psutil.disk_usage("/").percent
        inte_send = psutil.net_io_counters().bytes_sent / 1000000
        inte_recv = psutil.net_io_counters().bytes_recv / 1000000

        process = psutil.Process(os.getpid())
        b_cpu = process.cpu_percent(interval=1)
        b_mem = process.memory_percent(memtype="rss")

        now = time.time()
        boot = psutil.boot_time()
        b = process.create_time()
        boot_time = str(
            datetime.utcfromtimestamp(now).replace(microsecond=0)
            - datetime.utcfromtimestamp(boot).replace(microsecond=0)
        )
        bot_time = str(
            datetime.utcfromtimestamp(now).replace(microsecond=0)
            - datetime.utcfromtimestamp(b).replace(microsecond=0)
        )
    except Exception as e:
        return f"状态检查失败!\n{repr(e)}", False

    msg = "アトリは、高性能ですから！"
    if cpu > 90:
        msg = "咱感觉有些头晕..."
        is_ok = False
        if mem > 90:
            msg = "咱感觉有点头晕并且有点累..."
            is_ok = False
    elif mem > 90:
        msg = "咱感觉有点累..."
        is_ok = False
    elif disk > 90:
        msg = "咱感觉身体要被塞满了..."
        is_ok = False
    else:
        is_ok = True

    baidu_ = await get_url('https://www.baidu.com/')
    google_ = await get_url('https://www.google.com/')
        
    msg0 = _STATUS_MSG.format(
        p_cpu=cpu,
        p_mem=mem,
        p_disk=disk,
        b_cpu=b_cpu,
        b_mem="%.1f%%" % b_mem,
        baidu_=baidu_,
        google_=google_,
        inteSENT=inte_send,
        inteRECV=inte_recv,
        bot_time=bot_time,
        boot_time=boot_time,
        msg=msg,
    )

    return msg0, is_ok

headers = {
    'Referer': 'https://github.com/',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
}

async def get_url(url):
    async with AsyncClient() as client:
        response = await client.get(url, headers=headers, timeout=10)
        await response.aclose()
        return response.status_code




gua = (
    '䷁䷖䷇䷓䷏䷢䷬䷋'
    '䷎䷳䷦䷴䷽䷷䷞䷠'
    '䷆䷃䷜䷺䷧䷿䷮䷅'
    '䷭䷑䷯䷸䷟䷱䷛䷫'
    '䷗䷚䷂䷩䷲䷔䷐䷘'
    '䷣䷕䷾䷤䷶䷝䷰䷌'
    '䷒䷨䷻䷼䷵䷥䷹䷉'
    '䷊䷙䷄䷈䷡䷍䷪䷀'
)

from_bytes = int.from_bytes


def encode(s):
    """Encode the bytes-like object s using gua64 and return a bytes object.
    """
    encoded = bytearray()
    for i in range(0, len(s), 3):
        c = from_bytes(s[i: i + 3], 'big')
        if len(s[i: i + 3]) == 3:
            encoded += gua[c >> 18].encode()
            encoded += gua[(c >> 12) & 0x3f].encode()
            encoded += gua[(c >> 6) & 0x3f].encode()
            encoded += gua[c & 0x3f].encode()
            continue
        if len(s[i: i + 3]) == 2:
            encoded += gua[c >> 10].encode()
            encoded += gua[c >> 4 & 0x3f].encode()
            encoded += gua[(c & 0xf) << 2].encode()
            encoded += '☯'.encode()
            continue
        if len(s[i: i + 3]) == 1:
            encoded += gua[c >> 2].encode()
            encoded += gua[(c & 0x3) << 4].encode()
            encoded += '☯'.encode() * 2
    return bytes(encoded)


def decode(s):
    """Decode the bytes-like object s using gua64 and return a bytes object.
    """
    encoded = []
    gua64dict = {v: k for k, v in enumerate(gua)}
    for i in range(0, len(s), 3):
        if s[i: i + 3].decode() == '☯':
            encoded.append(0)
            continue
        encoded.append(gua64dict[s[i: i + 3].decode()])
    b = bytearray(encoded)
    encoded = []
    for i in range(0, len(b), 4):
        c = from_bytes(b[i: i + 4], 'big')
        if len(b[i: i + 4]) == 4:
            encoded.append(c >> 24 << 2 | (c >> 20 & 0x3))
            if (c >> 16 & 0xf) << 4 | (c >> 10 & 0xf) != 0:
                encoded.append((c >> 16 & 0xf) << 4 | (c >> 10 & 0xf))
            if (c >> 8 & 0x3) << 6 | (c & 0x3f) != 0:
                encoded.append((c >> 8 & 0x3) << 6 | (c & 0x3f))
    return bytes(encoded)


if __name__ == '__main__':
    r = encode('hello，世界'.encode())
    print(r.decode())

    r = decode('䷯䷬䷿䷶䷸䷬䷀䷌䷌䷎䷼䷲䷰䷳䷸䷘䷔䷭䷒☯'.encode())
    print(r.decode())



from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    filters
)

from remilia.log import logger
from .Gua64 import encode, decode


def run(application):
    en64Gua_handler = CommandHandler(
        'en64', en64Gua,
        filters.TEXT
    )
    application.add_handler(en64Gua_handler)

    de64Gua_handler = CommandHandler(
        'de64', de64Gua,
        filters.TEXT
    )
    application.add_handler(de64Gua_handler)


async def en64Gua(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = (
        context.args[0] + update.message.text.split(context.args[0], 1)[1]
        if context.args else ''
    )
    content = encode(msg.encode()).decode() if msg else '需要内容'
    await update.message.reply_text(content)


async def de64Gua(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = (
        context.args[0] + update.message.text.split(context.args[0], 1)[1]
        if context.args else ''
    )
    try:
        content = decode(msg.encode()).decode() if msg else '需要内容'
    except KeyError:
        content = '格式错误'
    await update.message.reply_text(content)




import random
from pathlib import Path
import ujson as json
from httpx import AsyncClient
from string import punctuation, whitespace

from remilia.log import logger
from remilia.config import XIAOAI


# 载入词库(这个词库有点涩)
AnimeThesaurus = json.loads(
    (Path(__file__).parent / 'resource' / 'data.json').read_text("utf-8")
)

# hello之类的回复
hello__reply = [
    'ʕ  •ᴥ•ʔ ?',
]

# 从字典里返还消息, 借鉴(抄)的zhenxun-bot
async def get_chat_result(text: str):
    if len(text) < 7:
        keys = AnimeThesaurus.keys()
        for key in keys:
            if text.find(key) != -1:
                return random.choice(AnimeThesaurus[key])


# 从思知api拿到消息
async def xiaosi(msg: str, NICKNAME: str) -> str:

    # 将半角标点和空白符号换成全角空格
    for i in punctuation + whitespace:
        msg = msg.replace(i, '　')

    url = f'https://api.ownthink.com/bot'
    params = {
        'appid': 'xiaosi',
        'userid': 'user',
        'spoken': msg,
    }
    headers = {
        'referer': 'https://www.ownthink.com/',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
    }

    async with AsyncClient() as client:
        try:
            response = await client.get(url=url, params=params, headers=headers)
            if response.json()['data']['type'] == 5000:
                res = response.json()['data']['info']['text'].replace('小思', NICKNAME)
                await response.aclose()
                return res
            return 'ʕ  •ᴥ•ʔ……'
        except Exception as e:
            logger.error(repr(e))
            return 'ʕ  •ᴥ•ʔ</>'


# 从小爱api拿到消息
async def xiaoai(msg: str, NICKNAME: str) -> str | bytes:

    # 将半角标点和空白符号换成全角空格
    for i in punctuation + whitespace:
        msg = msg.replace(i, '　')

    n: dict = XIAOAI['mp3']

    url = 'http://81.70.100.130/api/xiaoai.php'
    params = {
        'msg': msg,
        'n': 'mp3' if n else 'text',
    }
    headers = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
    }

    async with AsyncClient() as client:
        try:
            response = await client.get(url=url, params=params, headers=headers)
            res, status = response.text, response.status_code
            await response.aclose()

            if status == 200:
                content = (
                    await get_voice(res)
                    if n
                    else res.replace('小爱', NICKNAME).replace('小米智能助理', '猫')
                )
                return content if content else 'ʕ  •ᴥ•ʔ</>'
            logger.error(res)

        except Exception as e:
            logger.error(repr(e))
        return 'ʕ  •ᴥ•ʔ</>'


async def get_voice(url: str) -> bytes | None:

    headers = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36',
        'content-type': 'audio/mpeg',
    }
    async with AsyncClient() as client:
        try:
            response = await client.get(url=url, headers=headers, timeout=60)
            res, status = response.content, response.status_code
            await response.aclose()
            if status == 200:
                return res
            logger.error(res.decode())
        except Exception as e:
            logger.error(repr(e))




from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    filters
)

from remilia.src import scheduler
from remilia.log import logger
from remilia.config import SUPERUSERS
from .data_source import get_status


def run(application):
    status_handler = CommandHandler(
        'status', status,
        filters.User(SUPERUSERS)
    )
    application.add_handler(status_handler)


async def status(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg, _ = await get_status()
    await update.message.reply_text(msg)


@scheduler.scheduled_job(
    "interval",
    id="状态检查",
    name="状态检查",
    minutes=30,
    misfire_grace_time=15
)
async def _():
    logger.info("检查资源消耗中...")
    msg, stat = await get_status()
    if not stat:
        logger.warning(msg)
    else:
        logger.info("资源消耗正常")




import sys
from apscheduler.schedulers.asyncio import AsyncIOScheduler
from telegram.ext import ApplicationBuilder

from remilia.log import (
    logging,
    logger,
    LoguruHandler,
    default_format
)
from remilia.config import LOG_LEVEL, TOKEN


logging.basicConfig(handlers=[LoguruHandler()], level=logging.DEBUG)

logger.remove()
logger.add(sys.stdout, level=LOG_LEVEL, diagnose=False, format=default_format)


scheduler = AsyncIOScheduler(timezone="Europe/Moscow")
if not scheduler.running:
    try:
        scheduler.start()
    except Exception as e:
        logger.error(f"scheduler启动失败!\n{repr(e)}")


from .plugins import (
    block,
    chat,
    cmd,
    code_runner,
    Gua64,
    pix,
    setu,
    start,
    status
)

class run():
    app = ApplicationBuilder().token(TOKEN).build()

    start.run(app)
    block.run(app)
    cmd.run(app)
    status.run(app)
    code_runner.run(app)
    Gua64.run(app)
    pix.run(app)
    setu.run(app)

    chat.run(app)

    app.run_polling()



import random
from pathlib import Path
import ujson as json
from telegram import Update
from telegram.ext import (
    ContextTypes,
    CommandHandler,
    MessageHandler,
    filters
)

from remilia.log import logger
from remilia.config import SUPERUSERS, NICKNAME
from .utils import (
    xiaosi,
    xiaoai,
    get_chat_result,
    hello__reply
)
from .looklike import Look


confpath: Path = Path() / 'data' / 'smart_reply' / 'reply.json'
confpath.parent.mkdir(parents=True, exist_ok=True)

default_conf: dict = {'mode': 0}
conf: dict = (
    json.loads(confpath.read_text('utf-8'))
    if confpath.is_file() else default_conf
)
conf: dict = conf if conf.keys() == default_conf.keys() else default_conf


def save_conf() -> None:
    confpath.write_text(json.dumps(conf), encoding='utf-8')


def run(application):
    set_chat_handler = CommandHandler(
        'setreply', set_reply,
        filters.User(SUPERUSERS)
    )
    application.add_handler(set_chat_handler)

    chat_handler = MessageHandler(
        filters.ChatType.PRIVATE & filters.TEXT & (~filters.COMMAND),
        chat
    )
    application.add_handler(chat_handler)

    groupchat_handler = MessageHandler(
        filters.ChatType.GROUPS & filters.TEXT & (~filters.COMMAND),
        groupchat
    )
    application.add_handler(groupchat_handler)


async def set_reply(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = context.args[0] if context.args else ''

    if msg == '思知' or msg == '小思':
        conf['mode'] = 0
    elif msg == '小爱':
        conf['mode'] = 1
    elif not msg:
        conf['mode'] = 0 if conf['mode'] else 1
    else:
        await update.message.chat.send_message('模式不存在.')
        return

    save_conf()
    mode = ['小思', '小爱']
    await update.message.chat.send_message(f'已设置回复模式{mode[conf["mode"]]}')


async def chat(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = update.message.text.strip()
    content = await reply(msg, NICKNAME[0], context._user_id)
    if isinstance(content, str):
        await update.message.chat.send_message(content)
    else:
        await update.message.chat.send_voice(content)


async def groupchat(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = update.message.text
    for i in NICKNAME:
        if msg.startswith(i):
            msg = msg.replace(i, '', 1).strip()
            content = await reply(msg, NICKNAME[0], context._user_id)
            if isinstance(content, str):
                await update.message.chat.send_message(content)
            else:
                await update.message.chat.send_voice(content)


async def reply(msg: str, NICKNAME: str, uid: int):

    uid = uid

    if not conf['mode']:
        get_reply = xiaosi
    else:
        get_reply = xiaoai

    if not msg or msg in [
        "你好啊",
        "你好",
        "在吗",
        "在嗎",
        "在不在",
        "您好",
        "您好啊",
        "在",
    ]:
        return random.choice(hello__reply)
    if msg.startswith('你看我像'):
        return Look.like()
    # 从字典里获取结果
    result = await get_chat_result(msg)
    # 如果词库没有结果，则调用对话api获取回复
    if not result:
        content = await get_reply(msg, NICKNAME)
        return content
    return result

